@@ -31,8 +31,7 @@ def str_to_words(s):
 
 
 class AnyId:
-    """ Anything that has an id in a SpirV module
-    """
+    """Anything that has an id in a SpirV module"""
 
     def __init__(self, name=""):
         self.name = name
@@ -62,8 +61,7 @@ class AnyId:
 
 
 class TypeId(AnyId):
-    """ A type in a SpirV module.
-    """
+    """A type in a SpirV module."""
 
     def __init__(self, type, name=""):
         super().__init__(name=name)
@@ -71,17 +69,24 @@ class TypeId(AnyId):
 
 
 class ValueId(AnyId):
-    """ Anything that represents a concrete value in a SpirV module.
-    """
+    """Anything that represents a concrete value in a SpirV module."""
 
     def __init__(self, type, name=""):
         super().__init__(name=name)
         self.type = type
 
+    def clone(self, name=""):
+        assert self.id is not None
+        x = ValueId(self.type, name)
+        x.id = self.id
+        return x
+
 
 class VariableAccessId(ValueId):
-    """ A chain of access into a SpirV variable. The type arg is the type
-    for the eventual value.
+    """A chain of access into a SpirV variable. The type arg is the type
+    for the eventual value. This is a subclass of ValueId because it can usually
+    be used in place of one. However, it's only a wrapper class, and it's id
+    is always None.
     """
 
     def __init__(self, variable, storage_class, type, *indices, name=""):
@@ -89,42 +94,60 @@ class VariableAccessId(ValueId):
         self.variable = variable  # ValueId representing the SpirV Variable
         self.storage_class = storage_class
         self.indices = indices  # ValueId's
+        # self.id -> not used
+
+    def clone(self, name=""):
+        assert self.variable.id is not None
+        x = VariableAccessId(
+            self.variable, self.storage_class, self.type, *self.indices, name=name
+        )
+        return x
 
     def index(self, index, field=None):
-        """ Index into the variable chain, so we go one level deeper.
-        """
+        """Index into the variable chain, so we go one level deeper."""
         assert isinstance(index, ValueId)
         assert issubclass(index.type, _types.Int)
         indices = list(self.indices) + [index]
         if issubclass(self.type, _types.Struct):
             assert isinstance(field, int)
+            name = f"{self.name}.{self.type.keys[field]}" if self.name else ""
             return VariableAccessId(
                 self.variable,
                 self.storage_class,
                 self.type.get_subtype(field),
                 *indices,
+                name=name,
             )
         elif issubclass(self.type, _types.Array):
             assert field is None
+            name = f"{self.name}[{index.name or '..'}]" if self.name else ""
             return VariableAccessId(
-                self.variable, self.storage_class, self.type.subtype, *indices
+                self.variable,
+                self.storage_class,
+                self.type.subtype,
+                *indices,
+                name=name,
             )
         elif issubclass(self.type, _types.Vector):
             assert field is None
+            name = f"{self.name}[{index.name or '..'}]" if self.name else ""
             return VariableAccessId(
-                self.variable, self.storage_class, self.type.subtype, *indices
+                self.variable,
+                self.storage_class,
+                self.type.subtype,
+                *indices,
+                name=name,
             )
         else:
             raise ShaderError(f"VariableAccessId cannot index into {self.type}")
 
     def resolve_chain(self, gen):
-        """ Generate OpAccessChain instruction and return pointer id object for result.
-        """
+        """Generate OpAccessChain instruction and return pointer id object for result."""
         if len(self.indices) == 0:
             return self.variable
         else:
             result_type_id = gen.obtain_type_id(self.type)
-            pointer_id = gen.obtain_id("pointer")
+            pointer_id = gen.obtain_id()
             gen.gen_instruction(
                 "types",
                 cc.OpTypePointer,
@@ -132,23 +155,21 @@ class VariableAccessId(ValueId):
                 self.storage_class,
                 result_type_id,
             )
-            result_id = gen.obtain_id("access-chain-result")
+            result_id = gen.obtain_id()
             gen.gen_func_instruction(
                 cc.OpAccessChain, pointer_id, result_id, self.variable, *self.indices
             )
         return result_id
 
     def resolve_load(self, gen):
-        """ Generate OpAccessChain instruction followed by OpLoad and return result id.
-        """
+        """Generate OpAccessChain instruction followed by OpLoad and return result id."""
         temp_id = self.resolve_chain(gen)
-        id, type_id = gen.obtain_value(self.type)
+        id, type_id = gen.obtain_value(self.type, self.name)
         gen.gen_func_instruction(cc.OpLoad, type_id, id, temp_id)
         return id
 
     def resolve_store(self, gen, val):
-        """ Generate OpAccessChain instruction followed by OpStore.
-        """
+        """Generate OpAccessChain instruction followed by OpStore."""
         temp_id = self.resolve_chain(gen)
         gen.gen_func_instruction(cc.OpStore, temp_id, val)
         return val
@@ -158,7 +179,7 @@ class VariableAccessId(ValueId):
 
 
 class WordPlaceholder:
-    """ Object that holds an integer value (or 4 bytes), which value
+    """Object that holds an integer value (or 4 bytes), which value
     can be changed as more of the code is parsed. This e.g. allows
     specifying types with knowledge that we encounter later in the
     program.
@@ -173,14 +194,14 @@ class WordPlaceholder:
 
 
 class BaseSpirVGenerator:
-    """ Base class that can be used by compiler implementations in the
+    """Base class that can be used by compiler implementations in the
     last compile step to generate the SpirV code. It has an internal
     representation of SpirV module and provides an API to generate
     instructions. This class it not aware of our bytecode representation.
     """
 
     def convert(self, input):
-        """ Generate the Spir-V code. After this, dump() can be used to
+        """Generate the Spir-V code. After this, dump() can be used to
         produce the binary blob that represents the Spir-V module.
         """
 
@@ -194,8 +215,7 @@ class BaseSpirVGenerator:
         self._post_convert()
 
     def _convert(self, input):
-        """ Subclasses should implement this.
-        """
+        """Subclasses should implement this."""
         raise NotImplementedError()  # noqa
 
     def _init(self):
@@ -240,7 +260,7 @@ class BaseSpirVGenerator:
         }
 
     def _post_convert(self):
-        """ After most of the generation has been done, we set the required capabilities
+        """After most of the generation has been done, we set the required capabilities
         and massage the order of instructions a bit.
         """
 
@@ -320,8 +340,7 @@ class BaseSpirVGenerator:
     # %% Utility for compiler
 
     def to_text(self):
-        """ Generate a textual (dis-assembly-like) representation.
-        """
+        """Generate a textual (dis-assembly-like) representation."""
 
         lines = []
         edge = 22
@@ -368,8 +387,7 @@ class BaseSpirVGenerator:
         return "\n".join(lines)
 
     def dump(self):
-        """ Generated a bytes object representing the Spir-V module.
-        """
+        """Generated a bytes object representing the Spir-V module."""
 
         f = io.BytesIO()
 
@@ -414,32 +432,31 @@ class BaseSpirVGenerator:
     # %% Utils for subclasses
 
     def gen_instruction(self, section_name, opcode, *words):
-        words = [
-            word.resolve(self) if isinstance(word, AnyId) else word for word in words
-        ]
-        self._sections[section_name].append((opcode, *words))
+        # Resolve all args for this instruction
+        words_resolved = []
+        for word in words:
+            if isinstance(word, AnyId):
+                word = word.resolve(self)
+            words_resolved.append(word)
+        # Store
+        self._sections[section_name].append((opcode, *words_resolved))
 
     def gen_func_instruction(self, opcode, *words):
-        words = [
-            word.resolve(self) if isinstance(word, AnyId) else word for word in words
-        ]
-        self._sections["functions"].append((opcode, *words))
+        self.gen_instruction("functions", opcode, *words)
 
     def obtain_id(self, name=""):
-        """ Get a new raw id for anything that's not a value or type.
-        """
+        """Get a new raw id for anything that's not a value or type."""
         return AnyId(name=name)
 
     def obtain_value(self, the_type, name=""):
-        """ Create id for a new value. Returns (value_id, type_id).
-        """
+        """Create id for a new value. Returns (value_id, type_id)."""
         type_id = self.obtain_type_id(the_type)
-        value_id = ValueId(the_type)
+        value_id = ValueId(the_type, name)
         return value_id, type_id
         # todo: return only value, and support value.type_id?
 
     def obtain_constant(self, value, the_type=None):
-        """ Get the id object for the constant of given value.
+        """Get the id object for the constant of given value.
         Existing constants are re-used.
         """
         # First derive SpirV type from value
@@ -461,24 +478,26 @@ class BaseSpirVGenerator:
         # Make sure that we have it
         key = the_type.__name__, value
         if key not in self._constants:
-            id, type_id = self.obtain_value(the_type)
+            name = repr(value).lower()
+            id, type_id = self.obtain_value(the_type, name)
             if the_type is _types.boolean:
                 opcode = cc.OpConstantTrue if value else cc.OpConstantFalse
                 self.gen_instruction("types", opcode, type_id, id)
             else:
                 self.gen_instruction("types", cc.OpConstant, type_id, id, bb)
+            self.gen_instruction("debug", cc.OpName, id.id, name)
             self._constants[key] = id
         # Return cached
         return self._constants[key]
 
     def obtain_variable(self, the_type, storage_class, name=""):
-        """ Create a variable in the current scope. Generates an OpVariable
+        """Create a variable in the current scope. Generates an OpVariable
         definition instruction and returns a VariableAccessId to access it.
         """
         # Create id and type_id
         var_id, var_type_id = self.obtain_value(the_type, name)
         #  Create pointer for variable
-        var_pointer_id = self.obtain_id("pointer")
+        var_pointer_id = self.obtain_id()
         self.gen_instruction(
             "types", cc.OpTypePointer, var_pointer_id, storage_class, var_type_id
         )
@@ -489,11 +508,14 @@ class BaseSpirVGenerator:
         self.gen_instruction(
             where, cc.OpVariable, var_pointer_id, var_id, storage_class
         )
+        # Mark the name of this variable
+        if name:
+            self.gen_instruction("debug", cc.OpName, var_id.id, name)
         # Return object that can be used to access the variable with multi-level indexing
         return VariableAccessId(var_id, storage_class, the_type, name=name)
 
     def obtain_type_id(self, the_type):
-        """ Get the id for the given type. Generates a type
+        """Get the id for the given type. Generates a type
         definition instruction as needed.
         """
         if isinstance(the_type, tuple):
@@ -598,7 +620,7 @@ class BaseSpirVGenerator:
         return type_id
 
     def obtain_extended_instruction_set(self, set_name):
-        """ Obtain the extended instruction set object by the instruction set name.
+        """Obtain the extended instruction set object by the instruction set name.
         The used instruction sets are defined near the top of the SpirV file. The
         resulting id is used in OpExtInst instructions.
         """
