@@ -1,3 +1,4 @@
+import os
 import math
 import inspect
 from dis import dis as pprint_bytecode
@@ -15,7 +16,7 @@ EXTENDED_ARG = dis.opmap["EXTENDED_ARG"]
 
 
 def python2shader(func):
-    """ Convert a Python function to a ShaderModule object.
+    """Convert a Python function to a ShaderModule object.
 
     Takes the bytecode of the given function and converts it to our
     internal bytecode. From there it can be converted to binary SpirV.
@@ -45,16 +46,37 @@ def python2shader(func):
     return ShaderModule(func, bytecode, f"shader from {func.__name__}")
 
 
+def get_line_bumps_from_code_object(co):
+    """Get a list of tuples that define what instruction mark the beginning
+    of a new line.
+    """
+    # Convert Python's specific compressed format to a list of (absolute) tuples.
+    # https://svn.python.org/projects/python/branches/pep-0384/Objects/lnotab_notes.txt
+    lineno, addr = co.co_firstlineno, 0
+    line_bumps = []
+    for i in range(0, len(co.co_lnotab), 2):
+        addr_incr = co.co_lnotab[i]
+        line_incr = co.co_lnotab[i + 1]
+        addr += addr_incr
+        lineno += line_incr
+        if line_incr:
+            line_bumps.append((addr, lineno))
+    # Add an entry at the beginning (co_firstlineno may not match the first entry)
+    line_bumps.insert(0, (-1, co.co_firstlineno))
+    # Add an entry at the end, making processing easier.
+    line_bumps.append((len(co.co_code), line_bumps[-1][1]))
+    return line_bumps
+
+
 class PyBytecode2Bytecode:
-    """ Convert Python bytecode to our own well-defined bytecode.
+    """Convert Python bytecode to our own well-defined bytecode.
     Python bytecode depends on other variables on the code object, and differs
     between Python functions. This class converts this, so that the next step
     of code generation becomes simpler.
     """
 
     def show_bytecode(self):
-        """ For debugging purposes.
-        """
+        """For debugging purposes."""
         pprint_bytecode(self._co)
 
     def convert(self, py_func, shader_type):
@@ -68,6 +90,10 @@ class PyBytecode2Bytecode:
         self._co = self._py_func.__code__
         self._py_bytecode = self._co.co_code
 
+        # Get line bumps, and add an element for ease of use
+        self._line_bumps = get_line_bumps_from_code_object(self._co)
+        self._line_bump_index = len(self._line_bumps) - 2  # set at the last line
+
         self._opcodes = []  # The resulting "bytecode"
 
         self._input = {}
@@ -84,6 +110,9 @@ class PyBytecode2Bytecode:
         # and cannot be resolved if block is empty
         self._protected_labels = set()
 
+        # Code can insert instructions right before a certain target is handled
+        self._insert_at = {}  # int -> [instructions]
+
         # Bytecode is a stack machine.
         self._stack = []
 
@@ -93,6 +122,11 @@ class PyBytecode2Bytecode:
         # The loop_info objects are popped from the above lists and put on this stack
         self._loop_stack = [{}]  # prepend empty dict to be able to do get()
 
+        # Mark start or source (meta info for debugging)
+        # Note that the co_firstlineno may well point to the line "@python2shader"
+        self.emit(op.co_src_filename, self._co.co_filename)
+        self.emit(op.co_src_linenr, self._co.co_firstlineno)
+
         # todo: allow user to specify name otherwise?
         entrypoint_name = "main"  # py_func.__name__
         self.emit(op.co_entrypoint, entrypoint_name, shader_type, {})
@@ -148,10 +182,16 @@ class PyBytecode2Bytecode:
         self.emit(op.co_func_end)
 
     def _stack_pop(self, allow_global=False):
+        if not self._stack:
+            # Hacky fix for 3.8. Python normally does not have values
+            # on the stack between jumps, but 3.8 can in certain
+            # scenarios. I'm pretty sure that these scenarios are out
+            # of scope of PyShader anyway.
+            return
         ob = self._stack.pop()
         if not allow_global:
             if isinstance(ob, str) and ob.startswith("."):
-                raise ShaderError(f"Invalid use of (global) {ob[1:]}")
+                raise ShaderError(self.errinfo() + f"Invalid use of (global) {ob[1:]}")
         return ob
 
     def emit(self, opcode, *args):
@@ -161,18 +201,27 @@ class PyBytecode2Bytecode:
             argnames = [fcode.co_varnames[i] for i in range(fcode.co_argcount)][1:]
             if len(args) != len(argnames):
                 raise RuntimeError(
-                    f"Got {len(args)} args for {opcode}({', '.join(argnames)})"
+                    self.errinfo()
+                    + f"Got {len(args)} args for {opcode}({', '.join(argnames)})"
                 )
 
         if opcode == "co_branch":
             assert not self._opcodes[-1][0].startswith("co_branch")
+
         self._opcodes.append((opcode, *args))
 
+        if opcode == "co_src_linenr":
+            if self._opcodes[-2][0] == "co_label":
+                # Swap the last two items so that a co_src_linenr does not
+                # get in between a co_label and a co_branch
+                self._opcodes.insert(-1, self._opcodes.pop(-1))
+
     def dump(self):
         return self._opcodes
 
     def _convert(self):
 
+        self._line_bump_index = 0
         self._pointer = 0
         while self._pointer < len(self._py_bytecode):
             if (
@@ -195,13 +244,16 @@ class PyBytecode2Bytecode:
                 ):
                     self.emit(op.co_branch, label)
                 self.emit(op.co_label, label)
+            for instruction in self._insert_at.get(self._pointer, []):
+                self.emit(*instruction)
             opname, arg = self._next()
             method_name = "_op_" + opname.lower()
             method = getattr(self, method_name, None)
             if method is None:
                 pprint_bytecode(self._co)
                 raise RuntimeError(
-                    f"Cannot parse py's {opname} yet (no {method_name}())."
+                    self.errinfo()
+                    + f"Cannot parse py's {opname} yet (no {method_name}())."
                 )
             else:
                 method(arg)
@@ -240,8 +292,10 @@ class PyBytecode2Bytecode:
 
         self._pointer = 0
         while self._pointer < len(self._py_bytecode):
-            i = self._pointer
             opname, arg = self._next()
+            # Get index to this instruction. Do this after calling next(), because
+            # it may have jumped over EXTENDED_ARG instructions.
+            i = self._pointer - 2
             if "JUMP" in opname:
                 assert opname in jump_ops
                 jumps[i] = (i + 2 + arg) if opname == "JUMP_FORWARD" else arg
@@ -284,9 +338,9 @@ class PyBytecode2Bytecode:
 
         # Now we know the end (but there may be two positions to jump to)
         assert len(jumps_to_start) > 0
-        our_ends = [jumps_to_start[-1] + 2]
+        our_ends = [self._get_next_pos(jumps_to_start[-1], 2)]
         if self._peek(our_ends[0]) == "POP_BLOCK":
-            our_ends.append(our_ends[0] + 2)
+            our_ends.append(self._get_next_pos(our_ends[0], 2))
         ends = our_ends.copy()
         ends += [x["start"] for x in prev_loops] + [x["end"] for x in prev_loops]
 
@@ -297,10 +351,10 @@ class PyBytecode2Bytecode:
             if i > loop_start:
                 if target in ends:
                     first_jump_is_to_end = True
-                    body_target = i + 2
+                    body_target = self._get_next_pos(i, 2)
                 elif self._peek(target) == "BREAK_LOOP":
                     first_jump_is_to_end = True
-                    body_target = i + 2
+                    body_target = self._get_next_pos(i, 2)
                 break
 
         # Check what kind of loop this is
@@ -376,7 +430,7 @@ class PyBytecode2Bytecode:
         # Sometimes Python bytecode contains an empty block (i.e. code
         # jumpt to a location, from which it jumps to another location
         # immediately). In such cases, the control flow can be
-        # incosistent, with some branches jumping to that empty block,
+        # inconsistent, with some branches jumping to that empty block,
         # and some skipping it. The code below finds such empty blocks
         # and resolve them.
 
@@ -431,6 +485,10 @@ class PyBytecode2Bytecode:
                     # Register this branch (note that this may overwrite keys, which is ok)
                     conditional_branches[args[0]] = args[1], i
                     conditional_branches[args[1]] = args[0], i
+                elif "store" in opcode:
+                    # If there's any store ops here, this cannot have been an OR,
+                    # and we should not touch it, otherwise we break the flow.
+                    cur_block = None
 
         while True:
             block = _get_block_to_resolve()
@@ -442,6 +500,7 @@ class PyBytecode2Bytecode:
             labels2 = self._opcodes[i_cond][1:]  # the common block and the else
             # Rip out the current label
             selection = self._opcodes[i_label + 1 : i_cond]
+            # assert all(op[0] != "co_src_linenr" for op in selection
             self._opcodes[i_label : i_cond + 1] = []
             # Determine how to combine these
             if labels1[0] == labels2[0]:  # comp1 is true or comp2 is true
@@ -484,29 +543,54 @@ class PyBytecode2Bytecode:
 
         self._replace_labels(labels_to_replace)
 
+    def _get_next_pos(self, pos, delta):
+        # Move forward until we are at an actual instruction
+        if pos % 2 == 0:
+            while self._py_bytecode[pos] == EXTENDED_ARG:
+                pos += 2
+        else:
+            while self._py_bytecode[pos - 1] == EXTENDED_ARG:
+                pos += 2
+        if delta:
+            return self._get_next_pos(pos + 1, delta - 1)
+        return pos
+
     def _next(self):
         assert self._pointer % 2 == 0
+        # Skip over extended args first, easier to backtrack later
+        while self._py_bytecode[self._pointer] == EXTENDED_ARG:
+            self._pointer += 2
+        # Get opcode
         opcode = self._py_bytecode[self._pointer]
+        opname = dis.opname[opcode]
+        # Get arg value
         arg = self._py_bytecode[self._pointer + 1]
-        # Resolve name
-        opcode = dis.opname[opcode]
-        # Resolve EXTENDED_ARG
         n, i = 1, self._pointer
         while self._py_bytecode[i - 2] == EXTENDED_ARG:
             arg += self._py_bytecode[i - 1] * 256 ** n
             n += 1
             i -= 2
+        # Increase line number?
+        if self._pointer >= self._line_bumps[self._line_bump_index + 1][0]:
+            self._line_bump_index += 1
+            linenr = self._line_bumps[self._line_bump_index][1]
+            self.emit(op.co_src_linenr, linenr)
+        # Done
         self._pointer += 2
-        return opcode, arg
+        return opname, arg
 
-    def _peek(self, pos=None):
+    def _peek(self, pos=None, delta=0):
+        # Get initial position
         pos = self._pointer if pos is None else pos
-        res = self._py_bytecode[pos]
+        pos = self._get_next_pos(pos, delta)
+        # Now get the result, which is either the instruction, or its value
         if pos % 2 == 0:
             # Resolve name
+            res = self._py_bytecode[pos]
             res = dis.opname[res]
         else:
-            # Resolve EXTENDED_ARG
+            # Resolve value, taking EXTENDED_ARG into account
+            res = self._py_bytecode[pos]
             n, i = 1, pos - 1
             while self._py_bytecode[i - 2] == EXTENDED_ARG:
                 res += self._py_bytecode[i - 1] * 256 ** n
@@ -526,6 +610,37 @@ class PyBytecode2Bytecode:
             self._labels[pointer_pos] = str(pointer_pos)
         return self._labels[pointer_pos]
 
+    def errinfo(self, *variable_names):
+        """Get error info for the current moment during compiling
+        (source filename and line number) and including the names of the given
+        variables.
+        """
+        filename = self._co.co_filename
+        linenr = self._line_bumps[self._line_bump_index][1]
+        text = ""
+        # Start with basic info about the line
+        if filename:
+            text += f'\n  Source file "{filename}"'
+            if linenr:
+                text += f", line {linenr}"
+            text += ", in " + self._py_func.__name__
+            text += "\n"
+        # If possible, also add the line's source code
+        if filename and os.path.isfile(filename):
+            with open(filename, "rt", encoding="utf-8") as f:
+                lines = f.read().splitlines()
+            try:
+                text += "    " + lines[linenr - 1].strip() + "\n"
+            except IndexError:
+                pass
+        # Include variable names
+        if variable_names:
+            text += "  Related variables: " + ", ".join(variable_names) + "\n"
+        # Done
+        if text:
+            text += "  "
+        return text
+
     # %%
 
     def _op_extended_arg(self, arg):
@@ -623,19 +738,24 @@ class PyBytecode2Bytecode:
         elif isinstance(ob, tuple):
             if self._peek() != "UNPACK_SEQUENCE":
                 raise ShaderError(
-                    "Const tuples are not supported (though you can do `a, b = c, d`)"
+                    self.errinfo()
+                    + "Const tuples are not supported (though you can do `a, b = c, d`)"
                 )
             for x in ob:
                 if isinstance(x, (float, int, bool)):
                     self.emit(op.co_load_constant, x)
                     self._stack.append(x)
                 else:
-                    raise ShaderError("Only float/int/bool constants supported.")
+                    raise ShaderError(
+                        self.errinfo() + "Only float/int/bool constants supported."
+                    )
             self._stack.append(("tuple", len(ob)))  # signal for UNPACK_SEQUENCE
         elif ob is None:
             self._stack.append(None)  # Probably for the function return value
         else:
-            raise ShaderError("Only float/int/bool constants supported.")
+            raise ShaderError(
+                self.errinfo() + "Only float/int/bool constants supported."
+            )
 
     def _op_load_global(self, i):
         # Loading a global in Python can mean different things. We need
@@ -667,7 +787,7 @@ class PyBytecode2Bytecode:
             # Builtin functions that we resolve in this compiler
             self._stack.append(".py." + name)
         else:
-            raise ShaderError(f"Unknown variable name {name!r}")
+            raise ShaderError(self.errinfo() + f"Unknown variable name {name!r}")
         # todo: loading constants from the Python globals() scope
         # todo: loading other Python shader functions
 
@@ -680,7 +800,7 @@ class PyBytecode2Bytecode:
             self._stack.append(None)
         elif ob == ".stdlib":
             if name not in stdlib_func_names:
-                raise ShaderError(f"No stdlib function {name}")
+                raise ShaderError(self.errinfo() + f"No stdlib function {name}")
             self._stack.append(".stdlib." + name)  # new global on the stack
         elif ob.startswith(".math"):
             ob = getattr(math, name, None)
@@ -692,7 +812,7 @@ class PyBytecode2Bytecode:
             elif name in stdlib_func_names:
                 self._stack.append(".stdlib." + name)  # new global on the stack
             else:
-                raise ShaderError(f"No math constant/function {name}")
+                raise ShaderError(self.errinfo() + f"No math constant/function {name}")
         elif ob.startswith("texture."):
             # Calling a texture sampling function as a method on a texture
             # object. Not a global! We need to communicate to call_funcion/call_method
@@ -701,7 +821,9 @@ class PyBytecode2Bytecode:
             self._stack.append("texture." + name)
         elif ob.startswith("."):
             # Catch invalid use of globals
-            raise ShaderError(f"Cannot load attribute '{name}' from '{ob}'")
+            raise ShaderError(
+                self.errinfo() + f"Cannot load attribute '{name}' from '{ob}'"
+            )
         else:
             self.emit(op.co_load_attr, name)
             self._stack.append(None)
@@ -713,13 +835,13 @@ class PyBytecode2Bytecode:
     def _op_load_deref(self, arg):
         # ext_ob_name = self._co.co_freevars[i]
         # ext_ob = self._py_func.__closure__[i]
-        raise ShaderError("Shaders cannot be used as closures atm.")
+        raise ShaderError(self.errinfo() + "Shaders cannot be used as closures atm.")
 
     def _op_store_attr(self, i):
         name = self._co.co_names[i]
         ob = self._stack_pop()
         value = self._stack_pop()  # noqa
-        raise ShaderError(f"{ob}.{name} store")
+        raise ShaderError(self.errinfo() + f"{ob}.{name} store")
 
     def _op_call_function(self, nargs):
         args = [self._stack_pop(True) for i in range(nargs)]
@@ -728,7 +850,7 @@ class PyBytecode2Bytecode:
         func = self._stack_pop(True)  # allow global
 
         if not isinstance(func, str):
-            raise ShaderError(f"Cannot call object '{func}'.")
+            raise ShaderError(self.errinfo() + f"Cannot call object '{func}'.")
         self._call_function(func, args)
 
     def _op_call_method(self, nargs):
@@ -757,7 +879,7 @@ class PyBytecode2Bytecode:
                     args.append(arg)
         for arg in args:
             if isinstance(arg, str) and arg.startswith("."):
-                raise ShaderError(f"Cannot call {func} with arg {arg}")
+                raise ShaderError(self.errinfo() + f"Cannot call {func} with arg {arg}")
 
         if func.startswith("texture."):
             # A texture function called as a method of a texture object
@@ -781,11 +903,13 @@ class PyBytecode2Bytecode:
         elif func == ".py.range":
             if not (
                 self._peek(self._pointer) == "GET_ITER"
-                and self._peek(self._pointer + 2) == "FOR_ITER"
+                and self._peek(self._pointer, 2) == "FOR_ITER"
             ):
-                raise ShaderError("range() can only be used as a for-loop iter.")
+                raise ShaderError(
+                    self.errinfo() + "range() can only be used as a for-loop iter."
+                )
             loop_info = self._loops_to_handle[0]
-            assert loop_info["start"] == self._pointer + 2
+            assert loop_info["start"] == self._pointer + 2  # not _get_next_pos!
             loop_info["range_is_set"] = True
             if nargs == 1:
                 self.emit(op.co_load_constant, 0)
@@ -796,18 +920,20 @@ class PyBytecode2Bytecode:
             elif nargs == 3:
                 step = args[2]
                 if not (isinstance(step, int) and step > 0):
-                    raise ShaderError("range() step must be a constant int > 0")
+                    raise ShaderError(
+                        self.errinfo() + "range() step must be a constant int > 0"
+                    )
             else:
-                raise ShaderError("range() must have 1, 2 or 3 args.")
+                raise ShaderError(self.errinfo() + "range() must have 1, 2 or 3 args.")
             self._stack.append("range")
             # nothing to emit yet
         elif func.startswith((".stdlib.", ".math.")):
             self.emit(op.co_call, funcname, nargs)
             self._stack.append(None)
         elif func.startswith("."):
-            raise ShaderError(f"Unknown external function {func}.")
+            raise ShaderError(self.errinfo() + f"Unknown external function {func}.")
         else:
-            raise ShaderError(f"Cannot call object {func}.")
+            raise ShaderError(self.errinfo() + f"Cannot call object {func}.")
 
     def _op_binary_subscr(self, arg):
         index = self._stack_pop()
@@ -830,7 +956,8 @@ class PyBytecode2Bytecode:
             self._stack.append(("tuple", n))
         else:
             raise ShaderError(
-                "Tuples are not supported (though you can do `a, b = c, d`)"
+                self.errinfo()
+                + "Tuples are not supported (though you can do `a, b = c, d`)"
             )
 
     def _op_unpack_sequence(self, n):
@@ -842,10 +969,13 @@ class PyBytecode2Bytecode:
                 objects = [self._stack.pop() for i in range(n)]
                 self._stack.extend(objects)
             else:
-                raise ShaderError(f"Cannot unpack a {x[1]} tuple into a {n}-tuple")
+                raise ShaderError(
+                    self.errinfo() + f"Cannot unpack a {x[1]} tuple into a {n}-tuple"
+                )
         else:
             raise ShaderError(
-                "Cannot unpack arbitrary sequences (though you can do `a, b = c, d`)"
+                self.errinfo()
+                + "Cannot unpack arbitrary sequences (though you can do `a, b = c, d`)"
             )
 
     def _op_build_list(self, n):
@@ -856,11 +986,11 @@ class PyBytecode2Bytecode:
         self.emit(op.co_load_array, n)
 
     def _op_build_map(self, arg):
-        raise ShaderError("Dict not allowed in Shader-Python")
+        raise ShaderError(self.errinfo() + "Dict not allowed in Shader-Python")
 
     def _op_build_const_key_map(self, arg):
         # The version of BUILD_MAP specialized for constant keys. Py3.6+
-        raise ShaderError("Dict not allowed in Shader-Python")
+        raise ShaderError(self.errinfo() + "Dict not allowed in Shader-Python")
 
     def _op_unary_positive(self, arg):
         self._stack_pop()
@@ -947,7 +1077,9 @@ class PyBytecode2Bytecode:
     def _op_compare_op(self, arg):
         cmp = cmp_op[arg]
         if cmp not in ("<", "<=", "==", "!=", ">", ">="):
-            raise ShaderError(f"Compare op {cmp} not supported in shaders.")
+            raise ShaderError(
+                self.errinfo() + f"Compare op {cmp} not supported in shaders."
+            )
         self._stack_pop()
         self._stack_pop()
         self._stack.append(None)
@@ -955,9 +1087,16 @@ class PyBytecode2Bytecode:
 
     def _op_jump_absolute(self, target):
         label = self._get_label(target)
-        if label.startswith("Lm") and self._opcodes[-1][0] == "co_pop_top":
-            # This is a break in Python 3.8+ - I think it pops the iterator
-            self._opcodes.pop(-1)
+        if label.startswith("Lm") and target == self._loop_stack[-1]["end"]:
+            # This is a break in Python 3.8+
+            if self._opcodes[-1][0] == "co_pop_top":
+                # In Python bytecode, I think this is supposed to pop the iter
+                self._opcodes.pop(-1)
+            if self._peek() == "JUMP_ABSOLUTE":
+                # Python sometimes includes a bytecode that is never touched to jump
+                # to the beginning of the loop. Detect and ignore.
+                if self._peek(self._pointer, 1) == self._loop_stack[-1]["start"]:
+                    self._next()  # skip it
         self.emit(op.co_branch, label)
 
     def _op_jump_forward(self, delta):
@@ -989,29 +1128,22 @@ class PyBytecode2Bytecode:
         )
 
     def _op_jump_if_true_or_pop(self, target):
-        # This is xx OR yy, but only when a result is needed
-        # So not inside ``if xx or yy:``, but in ``if bool(xx or yy):``
+        # This is ``xx OR yy``, in some cases. Usually not in ``if xx or yy:``,
+        # but more commonly in e.g. ``zz = xx OR yy`` or in ``if (xx or yy):``
+        # In this case we can actually convert to a logical or (hooray!),
+        # which seems to mean that users can enforce using logical and/or
+        # by enclosing the condition of an if-statement in brackets.
 
         # The xx is now on the stack. In the next instructions yy will be
         # pushed on the stack, and at target, we continue. That's where we
         # need to insert the OR.
-
-        # self._insert_at[target] = ("co_binary_op", "or")
-
-        # ... except that determining if an arbitrary object is true
-        # or false is not trivial. We could add something like co_bool,
-        # but maybe we should avoid that temptation, as it does not fit
-        # a strongly typed language well ...
-        raise ShaderError(
-            "Implicit bool conversions not supported. Maybe use ``x if y else z``?"
-        )
+        self._stack_pop()
+        self._insert_at.setdefault(target, []).append(("co_binary_op", "or"))
 
     def _op_jump_if_false_or_pop(self, target):
         # Same as _op_jump_if_true_or_pop, but for AND
-        # self._insert_at[target] = ("co_binary_op", "and")
-        raise ShaderError(
-            "Implicit bool conversions not supported. Maybe use ``x if y else z``?"
-        )
+        self._stack_pop()
+        self._insert_at.setdefault(target, []).append(("co_binary_op", "and"))
 
     def _start_loop(self, loop_info):
 
@@ -1023,12 +1155,12 @@ class PyBytecode2Bytecode:
         if loop_info["type"] == "for":
             # Check that the range is set
             if not loop_info.get("range_is_set"):
-                raise ShaderError("Shader for-loop must use range()")
+                raise ShaderError(self.errinfo() + "Shader for-loop must use range()")
 
             # Consume next codepoint - the storing of the iter value
             assert self._peek(self._pointer) == "FOR_ITER"
-            assert self._peek(self._pointer + 2) == "STORE_FAST"
-            iter_name_index = self._peek(self._pointer + 2 + 1)
+            assert self._peek(self._pointer, 2) == "STORE_FAST"
+            iter_name_index = self._peek(self._pointer, 2 + 1)
             iter_name = self._co.co_varnames[iter_name_index]
             loop_info["iter_name"] = iter_name
 
@@ -1092,7 +1224,9 @@ class PyBytecode2Bytecode:
             # The continue_label and merge_label get emitted in _end_loop
 
         else:
-            raise RuntimeError(f"invalid loop type {loop_info['type'] }")
+            raise RuntimeError(
+                self.errinfo() + f"invalid loop type {loop_info['type'] }"
+            )
 
     def _end_loop(self):
 
@@ -1128,6 +1262,14 @@ class PyBytecode2Bytecode:
     def _op_break_loop(self, arg):
         # Python < 3.8
         self.emit(op.co_branch, self._loop_stack[-1]["merge_label"])
+        # Python sometimes includes a bytecode that is never touched to jump
+        # to the beginning of the loop (direct or indirectly). Detect and ignore.
+        if self._peek() == "JUMP_ABSOLUTE":
+            target = self._peek(self._pointer, 1)
+            while self._peek(target) == "JUMP_ABSOLUTE":
+                target = self._peek(target, 1)
+            if target == self._loop_stack[-1]["start"]:
+                self._next()  # skip it
 
     def _op_continue_loop(self, target):
         # This bytecode op is present in Python < 3.8, but does not seem to be
@@ -1140,7 +1282,7 @@ class PyBytecode2Bytecode:
     def _op_get_iter(self, arg):
         func = self._stack_pop()
         if func != "range":
-            raise ShaderError("Can only use a loop with range()")
+            raise ShaderError(self.errinfo() + "Can only use a loop with range()")
         self._stack.append(func)
         # Note: in op_call_function we've already made sure that there are three arg values on the stack
 
@@ -1155,7 +1297,7 @@ class PyBytecode2Bytecode:
         next_op, next_val = self._next()  # STORE_FAST, iter variable name
         loop_info = self._loop_stack[-1]
 
-        assert here == loop_info["start"]
+        assert here == self._get_next_pos(loop_info["start"], 0)
         assert target in (loop_info["end"], loop_info["end"] - 2)
         assert next_op == "STORE_FAST"
         assert self._co.co_varnames[next_val] == loop_info["iter_name"]
